/*
 *   Simple parallel port driver
 *
 *   By Sidney Li 5/23/95 
 *   compile with: gcc -c -I /usr/include/tcl ppout.c
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

/* for tcl/tk (of course!) */
#include <tcl.h>

/* FIFO monitoried by ppserver */
#define FIFO_FILE "/dev/PARAPORT"

/************************************************************/
 
int ppout_init ( Tcl_Interp *interp ){

extern int ib_PpOutCmd _ANSI_ARGS_(( ClientData clientData,
			      Tcl_Interp *interp,
			       int argc,
			       char *argv[]
			       ));

    Tcl_CreateCommand(interp,"ppout",ib_PpOutCmd,
		         (ClientData) NULL,
			 (Tcl_CmdDeleteProc *) NULL );
    return TCL_OK;
}


int ppout_cleanup ( Tcl_Interp *interp ){

  Tcl_DeleteCommand(interp,"ppout");

  return TCL_OK;
}

/**********************************************************************/

int
ib_PpOutCmd _ANSI_ARGS_(( ClientData clientData,
			      Tcl_Interp *interp,
			       int argc,
			       char *argv[]
			       )) 
{
  int outputbyte;
  FILE *fp;

  /* check number of arguments */
  if (argc != 2) {
      interp->result = "wrong # args; usage: ppout <number>";
      return TCL_ERROR;
  }

  /* get argument */
  if ( !isdigit(argv[1][0]) ){
      interp->result = "wrong arg type; usage: ppout <number>";
      return TCL_ERROR;
  }

  outputbyte = atoi(argv[1]);
#ifdef DEBUG
  printf("*** input is %s\n", argv[1]);
  printf("*** outputbyte is %d\n", outputbyte);
#endif

  if ( outputbyte < 0 || outputbyte > 255) {
      interp->result = "-1";
      return TCL_ERROR;
  }

  /* number is within range; now open FIFO */
  if ( (fp = fopen (FIFO_FILE, "w")) == NULL) {
      interp->result = "can't open parallel port";
      return TCL_ERROR;
  }

  fputs(argv[1], fp);
  fclose(fp);

  interp->result = argv[1];
  return TCL_OK;
}
