/*
 * ppserver -- a parallel port monitor:
 *
 * ppserver runs in the background to monitor the FIFO /dev/PARAPORT.
 * When another process sends a string to /dev/PARAPORT, ppserver
 * processes this to send an approriate output to the parallel port at
 * 0x378. 
 *
 * compile with: gcc -O2 -o ppserver ppserver.c
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <asm/io.h>
#include <unistd.h>

#include <linux/stat.h>

#define FIFO_FILE "/dev/PARAPORT"

int
openparaport()
{
  int test;
  int mem_fd;
  unsigned char *my_mem;

  if ( ioperm(0x378, 1, 1) ){
    printf("Port 0x378: Open failed! (Have to be root to open ports)\n");
    exit(1);
  }else{
    printf("Port 0x378: Open succeed!\n");
  }
}

int main (void)
{
  FILE *fp;
  char readbuf[10];
  int itmp, i;

#ifdef DEBUG
  int count = 0;
#endif

  /* initialize buffer */
  for (i=0 ; i<10; i++) {
    readbuf[i] = ' ';
  }
  /* create FIFO if not exists */
  umask(0);
  mknod(FIFO_FILE, S_IFIFO|0666, 0);

  /* initialize port 0x378 for output */
  openparaport();

  while(1){
    fp = fopen(FIFO_FILE,"r");
    fgets(readbuf,10,fp);

    /* The input has to be a number */
    if ( isdigit(readbuf[0]) ) {
      
      sscanf(readbuf,"%d",&itmp);
#ifdef DEBUG
      count ++;
#endif

      if(itmp < 0 || itmp > 255) {
#ifdef DEBUG
	printf("Port 0x378: error with input %d\n", itmp);
#endif
      }else{
	/* output to parallel port 0x378 */
	outb((unsigned char) itmp, 0x378); /* that's (value, port)! */
#ifdef DEBUG
	for(i = 0 ; i<10; i++) {
	  printf("!!! (%d) : %c\n", i, readbuf[i]);
	}
	printf("Port 0x378: input is %d, count is %d\n", itmp, count);  
#endif
      }
      fclose(fp);
    }
  }
  return(0);
}
